@@ -7,15 +7,16 @@ git config tar.tar.xz.command "xz -c"
 mkdir -p build && pushd build
 rm -rf *
 meson .. \
-    -Denable-doc=true \
-    -Denable-man=true \
-    -Denable-tests=true \
-    -Denable-dummy=true \
-    -Denable-thunderbolt=true \
-    -Denable-uefi=true \
-    -Denable-dell=true \
-    -Denable-synaptics=true \
-    -Denable-colorhug=true $@
+    -Dwerror=true \
+    -Dgtkdoc=true \
+    -Dman=true \
+    -Dtests=true \
+    -Dplugin_dummy=true \
+    -Dplugin_thunderbolt=true \
+    -Dplugin_uefi=true \
+    -Dplugin_dell=true \
+    -Dplugin_synaptics=true \
+    -Dplugin_colorhug=true $@
 ninja-build dist
 popd
 VERSION=`mesonintrospect build --projectinfo | jq -r .version`
@@ -34,14 +35,20 @@ sed "s,#VERSION#,$VERSION,;
 #build RPM packages
 rpmbuild -ba build/fwupd.spec
 
+#if invoked outside of CI
+if [ ! -f /.dockerenv ]; then
+        echo "Not running in a container, please manually install packages"
+        exit 0
+fi
+
 #install RPM packages
-dnf install -C -y $HOME/rpmbuild/RPMS/*/*.rpm
+dnf install -y $HOME/rpmbuild/RPMS/*/*.rpm
 
 cp $HOME/rpmbuild/RPMS/*/*.rpm .
 
 # run the installed tests
 if [ "$CI" = "true" ]; then
-	sed "s,^EnableTestSuite=.*,EnableTestSuite=true," -i /etc/fwupd.conf
+	sed "s,^BlacklistPlugins=test,BlacklistPlugins=," -i /etc/fwupd/daemon.conf
 	mkdir -p /run/dbus
 	mkdir -p /var
 	ln -s /var/run /run
