@@ -21,7 +21,7 @@
 
 #include "config.h"
 
-#include "fu-device-altos.h"
+#include "fu-altos-device.h"
 
 static void
 fu_altos_tool_write_progress_cb (goffset current, goffset total, gpointer user_data)
@@ -36,11 +36,9 @@ fu_altos_tool_write_progress_cb (goffset current, goffset total, gpointer user_d
 int
 main (int argc, char **argv)
 {
-
 	gsize len;
-	guint i;
 	g_autofree guint8 *data = NULL;
-	g_autoptr(FuDeviceAltos) dev = NULL;
+	g_autoptr(FuAltosDevice) dev = NULL;
 	g_autoptr(GBytes) fw = NULL;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GPtrArray) devices = NULL;
@@ -62,12 +60,12 @@ main (int argc, char **argv)
 	}
 	g_usb_context_enumerate (usb_ctx);
 	devices = g_usb_context_get_devices (usb_ctx);
-	for (i = 0; i < devices->len; i++) {
+	for (guint i = 0; i < devices->len; i++) {
 		GUsbDevice *usb_dev_tmp = g_ptr_array_index (devices, i);
-		g_autoptr(FuDeviceAltos) dev_tmp = fu_device_altos_new (usb_dev_tmp);
+		g_autoptr(FuAltosDevice) dev_tmp = fu_altos_device_new (usb_dev_tmp);
 		if (dev_tmp == NULL)
 			continue;
-		if (fu_device_altos_get_kind (dev_tmp) == FU_DEVICE_ALTOS_KIND_BOOTLOADER) {
+		if (fu_altos_device_get_kind (dev_tmp) == FU_ALTOS_DEVICE_KIND_BOOTLOADER) {
 			dev = g_object_ref (dev_tmp);
 			break;
 		}
@@ -79,10 +77,10 @@ main (int argc, char **argv)
 		return 1;
 	}
 	g_debug ("found %s",
-		 fu_device_altos_kind_to_string (fu_device_altos_get_kind (dev)));
+		 fu_altos_device_kind_to_string (fu_altos_device_get_kind (dev)));
 
 	/* open device */
-	if (!fu_device_altos_probe (dev, &error)) {
+	if (!fu_altos_device_probe (dev, &error)) {
 		g_print ("Failed to probe device: %s\n", error->message);
 		return 1;
 	}
@@ -96,8 +94,8 @@ main (int argc, char **argv)
 
 	/* update with data blob */
 	fw = g_bytes_new (data, len);
-	if (!fu_device_altos_write_firmware (dev, fw,
-					     FU_DEVICE_ALTOS_WRITE_FIRMWARE_FLAG_NONE,
+	if (!fu_altos_device_write_firmware (dev, fw,
+					     FU_ALTOS_DEVICE_WRITE_FIRMWARE_FLAG_NONE,
 					     fu_altos_tool_write_progress_cb, NULL,
 					     &error)) {
 		g_print ("Failed to write firmware: %s\n", error->message);
