@@ -428,7 +428,9 @@ fu_plugin_amt_create_device (GError **error)
 
 	dev = fu_device_new ();
 	fu_device_set_id (dev, "/dev/mei");
+	fu_device_set_vendor (dev, "Intel Corporation");
 	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_INTERNAL);
+	fu_device_add_icon (dev, "computer");
 	if (!amt_get_provisioning_state (ctx, &state, error))
 		return NULL;
 	switch (state) {
@@ -445,6 +447,8 @@ fu_plugin_amt_create_device (GError **error)
 		fu_device_set_name (dev, "Intel AMT (unknown)");
 		break;
 	}
+	fu_device_set_summary (dev, "Hardware and firmware technology for remote "
+				    "out-of-band management");
 
 	/* add guid */
 	memcpy (&uu, &ctx->guid, 16);
@@ -467,7 +471,7 @@ fu_plugin_amt_create_device (GError **error)
 }
 
 gboolean
-fu_plugin_startup (FuPlugin *plugin, GError **error)
+fu_plugin_coldplug (FuPlugin *plugin, GError **error)
 {
 	g_autoptr(FuDevice) dev = NULL;
 	dev = fu_plugin_amt_create_device (error);
