@@ -32,16 +32,6 @@ struct FuPluginData {
 	GUdevClient		*gudev_client;
 };
 
-//FIXME: this needs to move to the plugin core
-static gchar *
-fu_plugin_udev_get_id (GUdevDevice *device)
-{
-	gchar *id;
-	id = g_strdup_printf ("ro-%s", g_udev_device_get_sysfs_path (device));
-	g_strdelimit (id, "/:.-", '_');
-	return id;
-}
-
 gboolean
 fu_plugin_verify (FuPlugin *plugin,
 		  FuDevice *device,
@@ -71,7 +61,7 @@ fu_plugin_verify (FuPlugin *plugin,
 	if (g_strcmp0 (fu_device_get_version (device),
 		       fu_rom_get_version (rom)) != 0) {
 		g_debug ("changing version of %s from %s to %s",
-			 fu_device_get_id (device),
+			 fu_device_get_platform_id (device),
 			 fu_device_get_version (device),
 			 fu_rom_get_version (rom));
 		fu_device_set_version (device, fu_rom_get_version (rom));
@@ -131,9 +121,9 @@ fu_plugin_udev_add (FuPlugin *plugin, GUdevDevice *device)
 	FuDevice *dev_tmp;
 	const gchar *display_name;
 	const gchar *guid;
+	const gchar *id = NULL;
 	const gchar *product;
 	const gchar *vendor;
-	g_autofree gchar *id = NULL;
 	g_autofree gchar *rom_fn = NULL;
 	g_autofree gchar *vendor_id = NULL;
 	g_autofree gchar *version = NULL;
@@ -153,7 +143,7 @@ fu_plugin_udev_add (FuPlugin *plugin, GUdevDevice *device)
 	g_debug ("adding udev device: %s", g_udev_device_get_sysfs_path (device));
 
 	/* is already in database */
-	id = fu_plugin_udev_get_id (device);
+	id = g_udev_device_get_sysfs_path (device);
 	dev_tmp = fu_plugin_cache_lookup (plugin, id);
 	if (dev_tmp != NULL) {
 		g_debug ("ignoring duplicate %s", id);
@@ -174,8 +164,9 @@ fu_plugin_udev_add (FuPlugin *plugin, GUdevDevice *device)
 	/* did we get enough data */
 	dev = fu_device_new ();
 	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_INTERNAL);
-	fu_device_set_id (dev, id);
+	fu_device_set_platform_id (dev, id);
 	fu_device_add_guid (dev, guid);
+	fu_device_add_icon (dev, "audio-card");
 	display_name = g_udev_device_get_property (device, "FWUPD_MODEL");
 	if (display_name == NULL)
 		display_name = g_udev_device_get_property (device, "ID_MODEL_FROM_DATABASE");
@@ -215,7 +206,7 @@ fu_plugin_udev_remove (FuPlugin *plugin, GUdevDevice *device)
 		return;
 
 	/* already in database */
-	id = fu_plugin_udev_get_id (device);
+	id = g_udev_device_get_sysfs_path (device);
 	dev = fu_plugin_cache_lookup (plugin, id);
 	if (dev == NULL)
 		return;
@@ -242,7 +233,7 @@ void
 fu_plugin_init (FuPlugin *plugin)
 {
 	FuPluginData *data = fu_plugin_alloc_data (plugin, sizeof (FuPluginData));
-	const gchar *subsystems[] = { NULL };
+	const gchar *subsystems[] = { "pci", NULL };
 
 	data->gudev_client = g_udev_client_new (subsystems);
 	g_signal_connect (data->gudev_client, "uevent",
@@ -262,7 +253,7 @@ fu_plugin_coldplug (FuPlugin *plugin, GError **error)
 	FuPluginData *data = fu_plugin_get_data (plugin);
 	GList *devices;
 	GUdevDevice *udev_device;
-	const gchar *devclass[] = { "usb", "pci", NULL };
+	const gchar *devclass[] = { "pci", NULL };
 	g_autoptr(AsProfile) profile = as_profile_new ();
 
 	/* get all devices of class */
