@@ -30,6 +30,7 @@
 
 #include "lu-context.h"
 #include "lu-device.h"
+#include "lu-device-peripheral.h"
 
 struct FuPluginData {
 	LuContext		*ctx;
@@ -60,8 +61,16 @@ fu_plugin_unifying_device_added (FuPlugin *plugin,
 	dev = fu_device_new ();
 	if (lu_device_has_flag (device, LU_DEVICE_FLAG_CAN_FLASH))
 		fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_UPDATABLE);
-	fu_device_set_id (dev, lu_device_get_platform_id (device));
+	fu_device_set_platform_id (dev, lu_device_get_platform_id (device));
 	fu_device_set_name (dev, lu_device_get_product (device));
+	if (lu_device_get_kind (device) == LU_DEVICE_KIND_PERIPHERAL) {
+		const gchar *tmp;
+		tmp = lu_device_peripheral_get_summary (LU_DEVICE_PERIPHERAL (device));
+		if (tmp != NULL)
+			fu_device_set_summary (dev, tmp);
+	} else {
+		fu_device_set_summary (dev, "A miniaturised USB wireless receiver");
+	}
 	fu_device_set_vendor (dev, lu_device_get_vendor (device));
 	fu_device_set_vendor_id (dev, "USB:0x046D");
 	fu_device_set_version (dev, lu_device_get_version_fw (device));
@@ -72,6 +81,17 @@ fu_plugin_unifying_device_added (FuPlugin *plugin,
 		fu_device_add_guid (dev, guid);
 	}
 
+	/* add icon */
+	if (lu_device_get_kind (device) == LU_DEVICE_KIND_PERIPHERAL) {
+		const gchar *tmp = lu_device_peripheral_get_icon (LU_DEVICE_PERIPHERAL (device));
+		if (tmp != NULL)
+			fu_device_add_icon (dev, tmp);
+	} else {
+		/* FIXME: we need something better in the icon name spec for
+		 * the USB Unifying receiver dongle */
+		fu_device_add_icon (dev, "preferences-desktop-keyboard");
+	}
+
 	/* don't allow the USB plugin to claim this */
 	usb_device = lu_device_get_usb_device (device);
 	if (usb_device != NULL) {
@@ -103,7 +123,9 @@ fu_plugin_unifying_get_device (FuPlugin *plugin,
 			       GError **error)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
-	return lu_context_find_by_platform_id (data->ctx, fu_device_get_id (dev), error);
+	return lu_context_find_by_platform_id (data->ctx,
+					       fu_device_get_platform_id (dev),
+					       error);
 }
 
 static gboolean
@@ -138,6 +160,101 @@ fu_plugin_unifying_attach_cb (gpointer user_data)
 	return FALSE;
 }
 
+gboolean
+fu_plugin_update_detach (FuPlugin *plugin, FuDevice *dev, GError **error)
+{
+	FuPluginData *data = fu_plugin_get_data (plugin);
+	g_autoptr(LuDevice) device = NULL;
+
+	/* get device */
+	device = fu_plugin_unifying_get_device (plugin, dev, error);
+	if (device == NULL)
+		return FALSE;
+	if (!lu_device_open (device, error))
+		return FALSE;
+
+	/* switch to bootloader if required */
+	data->ignore_replug = TRUE;
+	if (!lu_device_has_flag (device, LU_DEVICE_FLAG_REQUIRES_DETACH))
+		return TRUE;
+
+	/* wait for device to come back */
+	fu_plugin_set_status (plugin, FWUPD_STATUS_DEVICE_RESTART);
+	if (lu_device_has_flag (device, LU_DEVICE_FLAG_DETACH_WILL_REPLUG)) {
+		g_debug ("doing detach in idle");
+		g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+				 fu_plugin_unifying_detach_cb,
+				 g_object_ref (device),
+				 (GDestroyNotify) g_object_unref);
+		if (!lu_context_wait_for_replug (data->ctx,
+						 device,
+						 FU_DEVICE_TIMEOUT_REPLUG,
+						 error))
+			return FALSE;
+	} else {
+		g_debug ("doing detach in main thread");
+		if (!lu_device_detach (device, error))
+			return FALSE;
+	}
+	return TRUE;
+}
+
+gboolean
+fu_plugin_update_attach (FuPlugin *plugin, FuDevice *dev, GError **error)
+{
+	FuPluginData *data = fu_plugin_get_data (plugin);
+	g_autoptr(LuDevice) device = NULL;
+
+	/* get device */
+	device = fu_plugin_unifying_get_device (plugin, dev, error);
+	if (device == NULL)
+		return FALSE;
+	if (!lu_device_open (device, error))
+		return FALSE;
+
+	/* wait for it to appear back in runtime mode if required */
+	if (!lu_device_has_flag (device, LU_DEVICE_FLAG_REQUIRES_ATTACH))
+		return TRUE;
+
+	/* wait for device to come back */
+	fu_plugin_set_status (plugin, FWUPD_STATUS_DEVICE_RESTART);
+	if (lu_device_has_flag (device, LU_DEVICE_FLAG_ATTACH_WILL_REPLUG)) {
+		g_debug ("doing attach in idle");
+		g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+				 fu_plugin_unifying_attach_cb,
+				 g_object_ref (device),
+				 (GDestroyNotify) g_object_unref);
+		if (!lu_context_wait_for_replug (data->ctx,
+						 device,
+						 FU_DEVICE_TIMEOUT_REPLUG,
+						 error))
+			return FALSE;
+	} else {
+		g_debug ("doing attach in main thread");
+		if (!lu_device_attach (device, error))
+			return FALSE;
+	}
+	data->ignore_replug = FALSE;
+	return TRUE;
+}
+
+gboolean
+fu_plugin_update_reload (FuPlugin *plugin, FuDevice *dev, GError **error)
+{
+	g_autoptr(LuDevice) device = NULL;
+
+	/* get device */
+	device = fu_plugin_unifying_get_device (plugin, dev, error);
+	if (device == NULL)
+		return FALSE;
+	if (!lu_device_open (device, error))
+		return FALSE;
+
+	/* set new version */
+	fu_device_set_version (dev, lu_device_get_version_fw (device));
+	return TRUE;
+}
+
 gboolean
 fu_plugin_update (FuPlugin *plugin,
 		  FuDevice *dev,
@@ -145,7 +262,6 @@ fu_plugin_update (FuPlugin *plugin,
 		  FwupdInstallFlags flags,
 		  GError **error)
 {
-	FuPluginData *data = fu_plugin_get_data (plugin);
 	g_autoptr(LuDevice) device = NULL;
 
 	/* get version */
@@ -155,73 +271,14 @@ fu_plugin_update (FuPlugin *plugin,
 	if (!lu_device_open (device, error))
 		return FALSE;
 
-	/* switch to bootloader */
-	data->ignore_replug = TRUE;
-	if (lu_device_has_flag (device, LU_DEVICE_FLAG_REQUIRES_DETACH)) {
-		/* wait for device to come back */
-		if (lu_device_has_flag (device, LU_DEVICE_FLAG_DETACH_WILL_REPLUG)) {
-			g_debug ("doing detach in idle");
-			g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-					 fu_plugin_unifying_detach_cb,
-					 g_object_ref (device),
-					 (GDestroyNotify) g_object_unref);
-			if (!lu_context_wait_for_replug (data->ctx,
-							 device,
-							 FU_DEVICE_TIMEOUT_REPLUG,
-							 error))
-				return FALSE;
-			g_object_unref (device);
-			device = fu_plugin_unifying_get_device (plugin, dev, error);
-			if (device == NULL)
-				return FALSE;
-			if (!lu_device_open (device, error))
-				return FALSE;
-		} else {
-			g_debug ("doing detach in main thread");
-			if (!lu_device_detach (device, error))
-				return FALSE;
-		}
-	}
-
 	/* write the firmware */
 	fu_plugin_set_status (plugin, FWUPD_STATUS_DEVICE_WRITE);
 	if (!lu_device_write_firmware (device, blob_fw,
 				       lu_write_progress_cb, plugin,
 				       error))
 		return FALSE;
-	fu_plugin_set_status (plugin, FWUPD_STATUS_DEVICE_RESTART);
-
-	/* wait for it to appear back in runtime mode */
-	if (lu_device_has_flag (device, LU_DEVICE_FLAG_REQUIRES_ATTACH)) {
-		if (lu_device_has_flag (device, LU_DEVICE_FLAG_ATTACH_WILL_REPLUG)) {
-			g_debug ("doing attach in idle");
-			g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-					 fu_plugin_unifying_attach_cb,
-					 g_object_ref (device),
-					 (GDestroyNotify) g_object_unref);
-			if (!lu_context_wait_for_replug (data->ctx,
-							 device,
-							 FU_DEVICE_TIMEOUT_REPLUG,
-							 error))
-				return FALSE;
-			g_object_unref (device);
-			device = fu_plugin_unifying_get_device (plugin, dev, error);
-			if (device == NULL)
-				return FALSE;
-			if (!lu_device_open (device, error))
-				return FALSE;
-		} else {
-			g_debug ("doing attach in main thread");
-			if (!lu_device_attach (device, error))
-				return FALSE;
-		}
-	}
-
-	/* set new version */
-	fu_device_set_version (dev, lu_device_get_version_fw (device));
 
 	/* success */
-	data->ignore_replug = FALSE;
 	return TRUE;
 }
 
@@ -278,12 +335,32 @@ gboolean
 fu_plugin_startup (FuPlugin *plugin, GError **error)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
+
+	/* check the kernel has CONFIG_HIDRAW */
+	if (!g_file_test ("/sys/class/hidraw", G_FILE_TEST_IS_DIR)) {
+		g_set_error_literal (error,
+				     FWUPD_ERROR,
+				     FWUPD_ERROR_NOT_SUPPORTED,
+				     "no kernel support for CONFIG_HIDRAW");
+		return FALSE;
+	}
+
+	/* coldplug */
 	g_signal_connect (data->ctx, "added",
 			  G_CALLBACK (fu_plugin_unifying_device_added_cb),
 			  plugin);
 	g_signal_connect (data->ctx, "removed",
 			  G_CALLBACK (fu_plugin_unifying_device_removed_cb),
 			  plugin);
+	lu_context_set_supported (data->ctx, fu_plugin_get_supported (plugin));
+	return TRUE;
+}
+
+
+gboolean
+fu_plugin_coldplug (FuPlugin *plugin, GError **error)
+{
+	FuPluginData *data = fu_plugin_get_data (plugin);
 	lu_context_coldplug (data->ctx);
 	lu_context_set_poll_interval (data->ctx, 5000);
 	return TRUE;
