@@ -39,26 +39,16 @@ struct _FuConfig
 	GPtrArray		*monitors;
 	GPtrArray		*blacklist_devices;
 	GPtrArray		*blacklist_plugins;
-	gboolean		 enable_option_rom;
 };
 
 G_DEFINE_TYPE (FuConfig, fu_config, G_TYPE_OBJECT)
 
-static const gchar *
-fu_config_get_sysconfig_dir (void)
-{
-	if (g_file_test (SYSCONFDIR, G_FILE_TEST_EXISTS))
-		return SYSCONFDIR;
-	return "/etc";
-}
-
 static GPtrArray *
 fu_config_get_config_paths (void)
 {
 	GPtrArray *paths = g_ptr_array_new_with_free_func (g_free);
 	const gchar *remotes_dir;
 	const gchar *system_prefixlibdir = "/usr/lib/fwupd";
-	const gchar *system_sysconfdir = "/etc/fwupd";
 	g_autofree gchar *sysconfdir = NULL;
 
 	/* only set by the self test program */
@@ -69,14 +59,9 @@ fu_config_get_config_paths (void)
 	}
 
 	/* use sysconfig, and then fall back to /etc */
-	sysconfdir = g_build_filename (SYSCONFDIR, "fwupd", NULL);
-	if (g_file_test (sysconfdir, G_FILE_TEST_EXISTS)) {
+	sysconfdir = g_build_filename (FWUPDCONFIGDIR, NULL);
+	if (g_file_test (sysconfdir, G_FILE_TEST_EXISTS))
 		g_ptr_array_add (paths, g_steal_pointer (&sysconfdir));
-	} else {
-		g_debug ("falling back to system path");
-		if (g_file_test (system_sysconfdir, G_FILE_TEST_EXISTS))
-			g_ptr_array_add (paths, g_strdup (system_sysconfdir));
-	}
 
 	/* add in system-wide locations */
 	if (g_file_test (system_prefixlibdir, G_FILE_TEST_EXISTS))
@@ -151,11 +136,19 @@ fu_config_add_remotes_for_path (FuConfig *self, const gchar *path, GError **erro
 		g_autofree gchar *filename = g_build_filename (path_remotes, tmp, NULL);
 		g_autoptr(FwupdRemote) remote = fwupd_remote_new ();
 
+		/* skip invalid files */
+		if (!g_str_has_suffix (tmp, ".conf")) {
+			g_debug ("skipping invalid file %s", filename);
+			continue;
+		}
+
 		/* load from keyfile */
-		g_debug ("loading from %s", filename);
+		g_debug ("loading config from %s", filename);
 		if (!fwupd_remote_load_from_filename (remote, filename,
-						      NULL, error))
+						      NULL, error)) {
+			g_prefix_error (error, "failed to load %s: ", filename);
 			return FALSE;
+		}
 
 		/* watch the config file and the XML file itself */
 		if (!fu_config_add_inotify (self, filename, error))
@@ -299,8 +292,7 @@ fu_config_load (FuConfig *self, GError **error)
 	g_ptr_array_set_size (self->remotes, 0);
 
 	/* load the main daemon config file */
-	config_file = g_build_filename (fu_config_get_sysconfig_dir (),
-					"fwupd.conf", NULL);
+	config_file = g_build_filename (FWUPDCONFIGDIR, "daemon.conf", NULL);
 	g_debug ("loading config values from %s", config_file);
 	if (!g_key_file_load_from_file (self->keyfile, config_file,
 					G_KEY_FILE_NONE, error))
@@ -315,13 +307,6 @@ fu_config_load (FuConfig *self, GError **error)
 			  G_CALLBACK (fu_config_monitor_changed_cb), self);
 	g_ptr_array_add (self->monitors, monitor);
 
-	/* optional, at the moment */
-	self->enable_option_rom =
-		g_key_file_get_boolean (self->keyfile,
-					"fwupd",
-					"EnableOptionROM",
-					NULL);
-
 	/* get blacklisted devices */
 	devices = g_key_file_get_string_list (self->keyfile,
 					      "fwupd",
@@ -352,13 +337,6 @@ fu_config_load (FuConfig *self, GError **error)
 	if (!fu_config_load_remotes (self, error))
 		return FALSE;
 
-	/* enable the test suite */
-	if (g_key_file_get_boolean (self->keyfile,
-				    "fwupd",
-				    "EnableTestSuite",
-				    NULL))
-		g_setenv ("FWUPD_TESTS", "true", TRUE);
-
 	/* success */
 	return TRUE;
 }
@@ -391,13 +369,6 @@ fu_config_get_blacklist_plugins (FuConfig *self)
 	return self->blacklist_plugins;
 }
 
-gboolean
-fu_config_get_enable_option_rom (FuConfig *self)
-{
-	g_return_val_if_fail (FU_IS_CONFIG (self), FALSE);
-	return self->enable_option_rom;
-}
-
 static void
 fu_config_class_init (FuConfigClass *klass)
 {
