@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2016 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2016-2017 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -22,17 +22,33 @@
 #ifndef __FU_PLUGIN_PRIVATE_H
 #define __FU_PLUGIN_PRIVATE_H
 
+#include "fu-quirks.h"
 #include "fu-plugin.h"
+#include "fu-smbios.h"
 
 G_BEGIN_DECLS
 
 #define FU_OFFLINE_TRIGGER_FILENAME	FU_OFFLINE_DESTDIR "/system-update"
 
 FuPlugin	*fu_plugin_new				(void);
+gboolean	 fu_plugin_has_device_delay		(FuPlugin	*plugin);
 void		 fu_plugin_set_usb_context		(FuPlugin	*plugin,
 							 GUsbContext	*usb_ctx);
 void		 fu_plugin_set_hwids			(FuPlugin	*plugin,
 							 FuHwids	*hwids);
+void		 fu_plugin_set_supported		(FuPlugin	*plugin,
+							 GPtrArray	*supported_guids);
+void		 fu_plugin_set_quirks			(FuPlugin	*plugin,
+							 FuQuirks	*quirks);
+void		 fu_plugin_set_smbios			(FuPlugin	*plugin,
+							 FuSmbios	*smbios);
+guint		 fu_plugin_get_order			(FuPlugin	*plugin);
+void		 fu_plugin_set_order			(FuPlugin	*plugin,
+							 guint		 order);
+void		 fu_plugin_set_name			(FuPlugin	*plugin,
+							 const gchar 	*name);
+GPtrArray	*fu_plugin_get_rules			(FuPlugin	*plugin,
+							 FuPluginRule	 rule);
 gboolean	 fu_plugin_open				(FuPlugin	*plugin,
 							 const gchar	*filename,
 							 GError		**error);
@@ -50,6 +66,18 @@ gboolean	 fu_plugin_runner_update_prepare	(FuPlugin	*plugin,
 gboolean	 fu_plugin_runner_update_cleanup	(FuPlugin	*plugin,
 							 FuDevice	*device,
 							 GError		**error);
+gboolean	 fu_plugin_runner_update_attach		(FuPlugin	*plugin,
+							 FuDevice	*device,
+							 GError		**error);
+gboolean	 fu_plugin_runner_update_detach		(FuPlugin	*plugin,
+							 FuDevice	*device,
+							 GError		**error);
+gboolean	 fu_plugin_runner_update_reload		(FuPlugin	*plugin,
+							 FuDevice	*device,
+							 GError		**error);
+gboolean	 fu_plugin_runner_usb_device_added	(FuPlugin	*plugin,
+							 GUsbDevice	*usb_device,
+							 GError		**error);
 void		 fu_plugin_runner_device_register	(FuPlugin	*plugin,
 							 FuDevice	*device);
 gboolean	 fu_plugin_runner_update		(FuPlugin	*plugin,
