@@ -55,12 +55,24 @@ gboolean	 fu_plugin_clear_results		(FuPlugin	*plugin,
 gboolean	 fu_plugin_get_results			(FuPlugin	*plugin,
 							 FuDevice	*dev,
 							 GError		**error);
+gboolean	 fu_plugin_update_attach		(FuPlugin	*plugin,
+							 FuDevice	*dev,
+							 GError		**error);
+gboolean	 fu_plugin_update_detach		(FuPlugin	*plugin,
+							 FuDevice	*dev,
+							 GError		**error);
+gboolean	 fu_plugin_update_reload		(FuPlugin	*plugin,
+							 FuDevice	*dev,
+							 GError		**error);
 gboolean	 fu_plugin_update_prepare		(FuPlugin	*plugin,
 							 FuDevice	*dev,
 							 GError		**error);
 gboolean	 fu_plugin_update_cleanup		(FuPlugin	*plugin,
 							 FuDevice	*dev,
 							 GError		**error);
+gboolean	 fu_plugin_usb_device_added		(FuPlugin	*plugin,
+							 GUsbDevice	*usb_device,
+							 GError		**error);
 void		 fu_plugin_device_registered		(FuPlugin	*plugin,
 							 FuDevice	*dev);
 
